<?php

namespace App\Installation\Commands;

use App\Installation\Commands\Contracts\Command;
use BadMethodCallException;
use Illuminate\Support\Facades\Storage;
use Symfony\Component\Process\Process;

/**
 * @mixin Process
 */
class CustomCommand implements Command
{
    /**
     * @var array
     */
    private $command;

    /**
     * @var \Symfony\Component\Process\Process
     */
    private $process;

    /**
     * @var array
     */
    private $options = [];

    public function __construct(string $command)
    {
        $this->command = explode(' ', $command);
    }

    public function getCommand(): string
    {
        return implode(' ', $this->buildCommand());
    }

    public function execute(): bool
    {
        $this->process = new Process(
            $this->buildCommand(),
            Storage::disk('project')->path('')
        );

        fwrite(STDOUT, PHP_EOL);
        $this->process->setTty(true)->run();
        fwrite(STDOUT, PHP_EOL);

        return $this->process->getExitCode() === 0;
    }

    public function options(array $options): self
    {
        $this->options = $options;

        return $this;
    }

    public function __call($name, $arguments)
    {
        if ($this->process === null) {
            throw new BadMethodCallException("The method {$name} does not exist");
        }

        return $this->process->$name(...$arguments);
    }

    private function buildCommand(): array
    {
        return array_merge($this->command, $this->options);
    }
}
