<?php

namespace App\Installation\Commands;

use App\Installation\Commands\Concerns\DependsOnNpm;
use Illuminate\Support\Facades\Storage;

class NpmBuild extends CustomCommand
{
    use DependsOnNpm;

    public function __construct()
    {
        parent::__construct($this->buildCommand());
    }

    private function buildCommand(): ?string
    {
        if (Storage::disk('project')->exists('package.json') === false) {
            return null;
        }

        $packageJson = json_decode(Storage::disk('project')->get('package.json'), false);

        if ($this->hasFailedToDecode($packageJson)) {
            return null;
        }

        $buildScript = $this->findBuildScript($packageJson);

        if ($buildScript === null) {
            return null;
        }

        return $this->npm() . ' run ' . $buildScript;
    }

    private function hasFailedToDecode($packageJson): bool
    {
        return $packageJson === null;
    }

    private function findBuildScript($packageJson): ?string
    {
        $scripts = $packageJson->scripts ?? (object)[];

        if (isset($scripts->dev)) {
            return 'dev';
        }

        if (isset($scripts->build)) {
            return 'build';
        }

        return null;
    }
}
