<?php

namespace App\Providers;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\ServiceProvider;
use RuntimeException;

class AppServiceProvider extends ServiceProvider
{
    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        $directory = $this->homeDirectory() . DIRECTORY_SEPARATOR . '.drp';

        if (is_dir($directory) === false) {
            if (!mkdir($directory) && !is_dir($directory)) {
                throw new RuntimeException(
                    sprintf('Directory "%s" was not created', $directory)
                );
            }
        }

        Storage::set(
            'home',
            Storage::createLocalDriver(['root' => $directory, 'links' => 'skip'])
        );

        Storage::set(
            'git-clone',
            Storage::createLocalDriver(['root' => $directory . DIRECTORY_SEPARATOR . 'projects', 'links' => 'skip'])
        );

        Storage::disk('home')->makeDirectory('storage');
        Storage::disk('home')->makeDirectory('cache');

        config([
            'cache.default' => 'home',
            'cache.stores.home' => [
                'driver' => 'file',
                'path' => Storage::disk('home')->path('cache'),
            ]
        ]);

        Command::macro('clear', function () {
            if ('\\' === DIRECTORY_SEPARATOR) {
                return system('cls');
            }

            return system('clear');
        });
    }

    /**
     * Return the user's home directory.
     */
    private function homeDirectory() {
        // Cannot use $_SERVER superglobal since that's empty during UnitUnishTestCase
        // getenv('HOME') isn't set on Windows and generates a Notice.
        $home = getenv('HOME');
        if (!empty($home)) {
            // home should never end with a trailing slash.
            $home = rtrim($home, '/');
        } elseif (!empty($_SERVER['HOMEDRIVE']) && !empty($_SERVER['HOMEPATH'])) {
            // home on windows
            $home = $_SERVER['HOMEDRIVE'] . $_SERVER['HOMEPATH'];
            // If HOMEPATH is a root directory the path can end with a slash. Make sure
            // that doesn't happen.
            $home = rtrim($home, '\\/');
        }
        return empty($home) ? NULL : $home;
    }

    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        //
    }
}
